/**
 * @file MX3080.cpp
 * @brief The implementation of class MX3080.
 */
#include "MX3080.h"

#define LOG_OUTPUT_DEBUG            (1)
#define LOG_OUTPUT_DEBUG_PREFIX     (1)

#define logDebug(arg)\
    do {\
        if (LOG_OUTPUT_DEBUG)\
        {\
            if (LOG_OUTPUT_DEBUG_PREFIX)\
            {\
                Serial.print("[LOG Debug: ");\
                Serial.print((const char*)__FILE__);\
                Serial.print(",");\
                Serial.print((unsigned int)__LINE__);\
                Serial.print(",");\
                Serial.print((const char*)__FUNCTION__);\
                Serial.print("] ");\
            }\
            Serial.print(arg);\
        }\
    } while(0)

MX3080::MX3080(SoftwareSerial &uart, uint32_t baud, ProcessCommand cb): m_puart(&uart)
{
    wifiString.reserve(100);
    m_puart->begin(baud);
    processCommand = cb;
    rx_empty();
}

int MX3080::print(const String &s)
{
    return m_puart->print(s);
}

int MX3080::println(const String &s)
{
    return m_puart->println(s);
}

int MX3080::available(void) {
    return m_puart->available();
}

int MX3080::read() {
    return m_puart->read();
}

int16_t* MX3080::hex2int16(char* string)
{
    if (string == NULL)
        return NULL;

    size_t slength = strlen(string);

    if ((slength % 4) != 0) // must be even
        return NULL;
    size_t dlength = slength / 4;
    int16_t *data = (int16_t *)malloc(dlength*2);
    memset(data, 0, dlength*2);

    size_t index = 0;
    while (index < slength) {
        char c = string[index];
        int value = 0;
        if (c >= '0' && c <= '9')
            value = (c - '0');
        else if (c >= 'A' && c <= 'F')
            value = (10 + (c - 'A'));
        else if (c >= 'a' && c <= 'f')
            value = (10 + (c - 'a'));
        else {
            free(data);
            return NULL;
        }
        int shift = 0;
        shift = 16 - (index + 1) % 4 * 4;
        shift = (shift == 16) ? 0 : shift;
        data[(index / 4)] += value << shift;
        index++;
    }
    return data;
}

void MX3080::readIfAvailable(void)
{
  if (available()) {
    char inChar = (char)read();
    if (wifiString.length() == 0 && isspace(inChar) != 0) {
        // 첫글자가 공백이면 skip
        wifiStringComplete = false;
    } else {
        if (inChar == '\r' || inChar == '\n') {
            wifiStringComplete = true;
        } else {
            wifiString += inChar;
        }
    }
  }
}

void MX3080::eventLooper(void) {
  readIfAvailable();

  if (!wifiStringComplete) return;
  handleWifiAction(wifiString);
  wifiString = "";
  wifiStringComplete = false;
}

void MX3080::handleWifiAction(const String &wifiMsg)
{
  if (!wifiMsg.startsWith("+")) return;
  if (wifiMsg.startsWith(F("+OK="))) { // +OK=0,0,0
    if (wifiMsg.length() < 9) return;
    String csv = wifiMsg.substring(4, wifiMsg.length()); //"+OK=".length
    int csvIdx = csv.lastIndexOf(",");
    if (csvIdx == -1) return;
    channel = csv.substring(csvIdx+1).toInt();
    csv.remove(csvIdx);
    csvIdx = csv.lastIndexOf(",");
    if (csvIdx == -1) return;
    rssi = csv.substring(csvIdx+1).toInt();
    shouldUpdateDevice = true;
    return;
  }
  if (wifiMsg.startsWith(F("+IOTREQSTAT"))) {
      shouldSendReport = true;
      return;
  }
  if (wifiMsg.startsWith(F("+IOTCMD="))) {
    String csv = wifiMsg.substring(8, wifiMsg.length()); //"+IOTCMD=".length
    while (csv.length() >= 3) { // "k:v".length
        int csvIdx = csv.lastIndexOf(",");
        String kv;
        if (csvIdx == -1 ) {
            kv = csv;
            processCommand(kv);
            csv.remove(0);
        } else {
            kv = csv.substring(csvIdx+1);
            processCommand(kv);
            csv.remove(csvIdx);
        }
    }
    shouldSendReport = true;
    shouldUpdateDevice = true;
    return;
  }
}

void MX3080::kick(void)
{
    m_puart->println(F("AT"));
}

void MX3080::restart(void)
{
    m_puart->println(F("AT+REBOOT"));
}

int MX3080::getRssi(void)
{
    m_puart->println(F("AT+WLINK"));
    return rssi;
}
void MX3080::skPair(void)
{
   m_puart->println(F("AT+SKPAIR"));
}

void MX3080::skStat(char *csv)
{
    m_puart->print(F("AT+SKSTAT="));
    m_puart->println(csv);
}

void MX3080::rx_empty(void)
{
    while(m_puart->available() > 0) {
        m_puart->read();
    }
}
